
import atexit
from itertools import chain
from functools import wraps

# If pandas is available, wrap talib functions so that they support
# pandas.Series input
try:
    from pandas import Series as _pd_Series
except ImportError:
    # pandas not available, nothing to wrap
    _pandas_wrapper = lambda x: x
else:
    def _pandas_wrapper(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                # Get the index of the first Series object, if any
                index = next(arg.index
                             for arg in chain(args, kwargs.values())
                             if isinstance(arg, _pd_Series))
            except StopIteration:
                # No pandas.Series passed in; short-circuit
                index = None

            if index is None:
                return func(*args, **kwargs)

            # Use Series' float64 values if pandas, else use values as passed
            args = [arg.values.astype(float) if isinstance(arg, _pd_Series) else arg
                    for arg in args]
            kwargs = {k: v.values.astype(float) if isinstance(v, _pd_Series) else v
                      for k, v in kwargs.items()}

            result = func(*args, **kwargs)

            # check to see if we got a streaming result
            first_result = result[0] if isinstance(result, tuple) else result
            is_streaming_fn_result = not hasattr(first_result, '__len__')
            if is_streaming_fn_result:
                return result

            # Series was passed in, Series gets out; re-apply index
            if isinstance(result, tuple):
                # Handle multi-array results such as BBANDS
                return tuple(_pd_Series(arr, index=index)
                             for arr in result)
            return _pd_Series(result, index=index)

        return wrapper

from ._ta_lib import (
    _ta_initialize, _ta_shutdown, MA_Type, __ta_version__,
    _ta_set_unstable_period as set_unstable_period,
    _ta_get_unstable_period as get_unstable_period,
    _ta_set_compatibility as set_compatibility,
    _ta_get_compatibility as get_compatibility,
    __TA_FUNCTION_NAMES__
)

# import all the func and stream functions
from ._ta_lib import *

# wrap them with pandas
func = __import__("_ta_lib", globals(), locals(), __TA_FUNCTION_NAMES__, level=1)
for func_name in __TA_FUNCTION_NAMES__:
    wrapped_func = _pandas_wrapper(getattr(func, func_name))
    setattr(func, func_name, wrapped_func)
    globals()[func_name] = wrapped_func

stream_func_names = ['stream_%s' % fname for fname in __TA_FUNCTION_NAMES__]
stream = __import__("stream", globals(), locals(), stream_func_names, level=1)
for func_name, stream_func_name in zip(__TA_FUNCTION_NAMES__, stream_func_names):
    wrapped_func = _pandas_wrapper(getattr(stream, func_name))
    setattr(stream, func_name, wrapped_func)
    globals()[stream_func_name] = wrapped_func

__version__ = '0.4.19'

# In order to use this python library, talib (i.e. this __file__) will be
# imported at some point, either explicitly or indirectly via talib.func
# or talib.abstract. Here, we handle initializing and shutting down the
# underlying TA-Lib. Initialization happens on import, before any other TA-Lib
# functions are called. Finally, when the python process exits, we shutdown
# the underlying TA-Lib.

_ta_initialize()
atexit.register(_ta_shutdown)

__function_groups__ = {
    'Cycle Indicators': [
        'HT_DCPERIOD',
        'HT_DCPHASE',
        'HT_PHASOR',
        'HT_SINE',
        'HT_TRENDMODE',
        ],
    'Math Operators': [
        'ADD',
        'DIV',
        'MAX',
        'MAXINDEX',
        'MIN',
        'MININDEX',
        'MINMAX',
        'MINMAXINDEX',
        'MULT',
        'SUB',
        'SUM',
        ],
    'Math Transform': [
        'ACOS',
        'ASIN',
        'ATAN',
        'CEIL',
        'COS',
        'COSH',
        'EXP',
        'FLOOR',
        'LN',
        'LOG10',
        'SIN',
        'SINH',
        'SQRT',
        'TAN',
        'TANH',
        ],
    'Momentum Indicators': [
        'ADX',
        'ADXR',
        'APO',
        'AROON',
        'AROONOSC',
        'BOP',
        'CCI',
        'CMO',
        'DX',
        'MACD',
        'MACDEXT',
        'MACDFIX',
        'MFI',
        'MINUS_DI',
        'MINUS_DM',
        'MOM',
        'PLUS_DI',
        'PLUS_DM',
        'PPO',
        'ROC',
        'ROCP',
        'ROCR',
        'ROCR100',
        'RSI',
        'STOCH',
        'STOCHF',
        'STOCHRSI',
        'TRIX',
        'ULTOSC',
        'WILLR',
        ],
    'Overlap Studies': [
        'BBANDS',
        'DEMA',
        'EMA',
        'HT_TRENDLINE',
        'KAMA',
        'MA',
        'MAMA',
        'MAVP',
        'MIDPOINT',
        'MIDPRICE',
        'SAR',
        'SAREXT',
        'SMA',
        'T3',
        'TEMA',
        'TRIMA',
        'WMA',
        ],
    'Pattern Recognition': [
        'CDL2CROWS',
        'CDL3BLACKCROWS',
        'CDL3INSIDE',
        'CDL3LINESTRIKE',
        'CDL3OUTSIDE',
        'CDL3STARSINSOUTH',
        'CDL3WHITESOLDIERS',
        'CDLABANDONEDBABY',
        'CDLADVANCEBLOCK',
        'CDLBELTHOLD',
        'CDLBREAKAWAY',
        'CDLCLOSINGMARUBOZU',
        'CDLCONCEALBABYSWALL',
        'CDLCOUNTERATTACK',
        'CDLDARKCLOUDCOVER',
        'CDLDOJI',
        'CDLDOJISTAR',
        'CDLDRAGONFLYDOJI',
        'CDLENGULFING',
        'CDLEVENINGDOJISTAR',
        'CDLEVENINGSTAR',
        'CDLGAPSIDESIDEWHITE',
        'CDLGRAVESTONEDOJI',
        'CDLHAMMER',
        'CDLHANGINGMAN',
        'CDLHARAMI',
        'CDLHARAMICROSS',
        'CDLHIGHWAVE',
        'CDLHIKKAKE',
        'CDLHIKKAKEMOD',
        'CDLHOMINGPIGEON',
        'CDLIDENTICAL3CROWS',
        'CDLINNECK',
        'CDLINVERTEDHAMMER',
        'CDLKICKING',
        'CDLKICKINGBYLENGTH',
        'CDLLADDERBOTTOM',
        'CDLLONGLEGGEDDOJI',
        'CDLLONGLINE',
        'CDLMARUBOZU',
        'CDLMATCHINGLOW',
        'CDLMATHOLD',
        'CDLMORNINGDOJISTAR',
        'CDLMORNINGSTAR',
        'CDLONNECK',
        'CDLPIERCING',
        'CDLRICKSHAWMAN',
        'CDLRISEFALL3METHODS',
        'CDLSEPARATINGLINES',
        'CDLSHOOTINGSTAR',
        'CDLSHORTLINE',
        'CDLSPINNINGTOP',
        'CDLSTALLEDPATTERN',
        'CDLSTICKSANDWICH',
        'CDLTAKURI',
        'CDLTASUKIGAP',
        'CDLTHRUSTING',
        'CDLTRISTAR',
        'CDLUNIQUE3RIVER',
        'CDLUPSIDEGAP2CROWS',
        'CDLXSIDEGAP3METHODS',
        ],
    'Price Transform': [
        'AVGPRICE',
        'MEDPRICE',
        'TYPPRICE',
        'WCLPRICE',
        ],
    'Statistic Functions': [
        'BETA',
        'CORREL',
        'LINEARREG',
        'LINEARREG_ANGLE',
        'LINEARREG_INTERCEPT',
        'LINEARREG_SLOPE',
        'STDDEV',
        'TSF',
        'VAR',
        ],
    'Volatility Indicators': [
        'ATR',
        'NATR',
        'TRANGE',
        ],
    'Volume Indicators': [
        'AD',
        'ADOSC',
        'OBV'
        ],
    }

def get_functions():
    """
    Returns a list of all the functions supported by TALIB
    """
    ret = []
    for group in __function_groups__:
        ret.extend(__function_groups__[group])
    return ret

def get_function_groups():
    """
    Returns a dict with keys of function-group names and values of lists
    of function names ie {'group_names': ['function_names']}
    """
    return __function_groups__.copy()

__all__ = ['get_functions', 'get_function_groups'] + __TA_FUNCTION_NAMES__ + ["stream_%s" % name for name in __TA_FUNCTION_NAMES__]



import time
from tcorepyapi.tcoreapi_mq import *
import pandas as pd
#登入
TCoreAPI= TCoreZMQ(quote_port="51630",trade_port="51600")
#TCoreAPI= TCoreZMQ(quote_port="51864",trade_port="51834")
#TCoreAPI= TCoreZMQ(quote_port="51424",trade_port="51394")
#查询指定合约信息
print("查询指定合约：",TCoreAPI.QueryInstrumentInfo("TC.O.SSE.510050.202108.C.3.6"))

#查询指定类型合约列表
#参数1：
    #期货：Future
    #期权：Options
    #证券：Stock
#参数2：指定日期的合约列表，如果不带或带入空参数，返回最新的合约列表，否则返回指定日期的合约列表
symbollist=TCoreAPI.QueryAllInstrumentInfo("Options","20210824")
#print("查询合约：\n",symbollist)
#解析期权合约列表
if symbollist['Success']=="OK":
    for opt in symbollist['Instruments']['Node']:
        #SSE交易所的期权
        if opt['ENG']=='SSE(O)':
            for optlist in opt['Node']:
                #标的50etf的期权
                if optlist['ENG_M']=='50ETF':
                    for optlist2 in optlist['Node']:
                        #202108月份合约
                        if optlist2['ENG']=='202108':
                            for syblist in optlist2['Node']:
                                for symb in syblist['Contracts']:
                                    print(symb)
#解除订阅
#TCoreAPI.UnsubQuote("TC.F.CFFEX.IF.202106")
#订阅实时行情
TCoreAPI.SubQuote("TC.O.SSE.510050.202108.C.3.2")
#解除订阅
#TCoreAPI.UnsubGreeks("TC.F.U_SSE.510050.202106")
#订阅实时Greeks行情
#TCoreAPI.SubGreeks("TC.F.U_SSE.510050.202106")

#获取历史数据
    #1：合约代码，
    #2：数据周期:
        ##标准行情历史数据
        # tick: "TICKS", 
        # 分K: "1K",
        # 日K: "DK"，
        ##DOGS为包含Greeks、标的期货、标的合约的历史数据
        # DOGS秒："DOGSS", 
        # DOGS分K："DOGSK"
    #3: 历史数据开始时间,
    #4: 历史数据结束时间
#his=TCoreAPI.SubHistory("TC.F.U_SSE.510050.202106", "DOGSK", "2021050100", "2021052107")
his=TCoreAPI.SubHistory("TC.O.SSE.510050.202108.C.3.6", "1K", "2021062300", "2021062307")
print("历史数据：\n",his)

#查询已登入资金账户
accountInfo = TCoreAPI.QryAccount()
#查询当日委托回报
reportData = TCoreAPI.QryReport()
if reportData:
    print("当日委托回报：\n",pd.DataFrame(reportData))
else:
    print("今日没有委托记录")
#查询当日成交回报
fillReportData = TCoreAPI.QryFillReport()
if fillReportData:
    print("当日成交回报：\n",pd.DataFrame(fillReportData))
else:
    print("今日没有成交记录")
#查询持仓监控
PositionTracker=TCoreAPI.QryPositionTracker()
if PositionTracker:
    print("持仓监控信息：\n",PositionTracker)
else:
    print("没有持仓监控信息")
strAccountMask =""
if accountInfo != None:
    arrInfo = accountInfo["Accounts"]
    if len(arrInfo) != 0:
        for acc in arrInfo:
            print("当前已登入资金账户\n",pd.DataFrame.from_dict(acc,orient='index').T)
        #获取账户列表中的第一个账户
        strAccountMask = arrInfo[0]["AccountMask"]
        #查询资金
        margin=TCoreAPI.QryMargin(strAccountMask)
        print("资金信息：\n",pd.DataFrame(margin["Margins"]))
        #查询持仓
        pos = TCoreAPI.QryPosition(strAccountMask)
        print("持仓信息：\n",pd.DataFrame(pos))

flag=True
while True:
    #行情更新
    quoteupdate= TCoreAPI.mdupdate()

    if quoteupdate:
        if quoteupdate['DataType']=='REALTIME':
            print("实时行情: \n",pd.DataFrame.from_dict(quoteupdate['Quote'],orient='index').T)  #这里只是展示用，实际应用建议不要使用dataframe转换实时数据，会影响运行效率
            if strAccountMask !="" and flag:
                #新增一笔委托
                orders_obj = {
                                "Symbol":quoteupdate['Quote']['Symbol'],
                                "BrokerID":arrInfo[0]['BrokerID'],
                                "Account":arrInfo[0]['Account'],
                                "Price":"0.0001",#quoteupdate['Quote']['LowPrice'],
                                "TimeInForce":"1",
                                "Side":"1",
                                "OrderType":"2",
                                "OrderQty":"1",
                                "PositionEffect":"4",
                                "SelfTradePrevention":"3"
                                #"ChasePrice":"1T|5|3|M"
                            }
                ordid = TCoreAPI.NewOrder(orders_obj)
                if ordid!=None:
                    while True:
                        if TCoreAPI.getorderinfo(ordid):
                            print("新增委托",TCoreAPI.getorderinfo(ordid)['ReportID'])#,TCoreAPI.QryReport()[-1]['ReportID'])
                            break

                #改单
                time.sleep(1)
                print(TCoreAPI.getorderinfo(ordid)['ExecType'])
                reporders_obj={
                    "ReportID":TCoreAPI.getorderinfo(ordid)['ReportID'],#TCoreAPI.QryReport()[-1]['ReportID'],
                    "ReplaceExecType":"0",
                    "Price":quoteupdate['Quote']['TradingPrice']
                    }
                #在可改单的委托单状态下发送改单指令（注意：模拟交易和外盘允许改单，内盘建议使用先删单单然后发送新委托方式改单）
                if TCoreAPI.getorderinfo(ordid)['ExecType'] in ["0","1","4","6"]:
                    reorder=TCoreAPI.ReplaceOrder(reporders_obj)
                    print("修改委托单：\n",reorder,TCoreAPI.QryReport()[-1]['ReportID'])
                #取消委托单
                time.sleep(1)
                cancalorders_obj = {
                            "ReportID":TCoreAPI.getorderinfo(ordid)['ReportID'],#TCoreAPI.QryReport()[-1]['ReportID'],
                            }
                #在可删单的委托单状态下发送删单指令      
                if TCoreAPI.getorderinfo(ordid)['ExecType'] in ["0","1","4","6"]:
                    cancal_order = TCoreAPI.CancelOrder(cancalorders_obj )
                    print("删单:",cancal_order)
                #建组合
                orders_obj = {
                            "Symbol":"TC.O.SSE.510050.202106.C.3.6&TC.O.SSE.510050.202106.C.3.7",
                            "BrokerID":arrInfo[0]['BrokerID'],
                            "Account":arrInfo[0]['Account'],
                            "CombDirection":"1",
                            "Side":"1",
                            "CombinationType":"1",
                            "Volume":"1"
                            }
                TCoreAPI.OptComb(orders_obj)
                #查询新建组合的委托回报
                TCoreAPI.QryOptCombOrder(strAccountMask,"")
                flag=False

        elif  quoteupdate['DataType']=="GREEKS":
            print("实时GREEKS: \n",pd.DataFrame.from_dict(quoteupdate['Quote'],orient='index').T)      
    #交易更新
    tradeupdate= TCoreAPI.tdupdate()
    if tradeupdate:
        if tradeupdate['DataType']=='ACCOUNTS':
            print("账户列表更新: \n",tradeupdate['Accounts'])
            arrInfo = tradeupdate['Accounts']
            if len(arrInfo) != 0:
                strAccountMask = arrInfo[0]["AccountMask"]
        elif tradeupdate['DataType'] == "EXECUTIONREPORT":
            print("实时委托回报: \n",pd.DataFrame.from_dict(tradeupdate["Report"],orient='index').T)
        elif tradeupdate['DataType'] == "FILLEDREPORT":
            print("实时成交: \n",pd.DataFrame.from_dict(tradeupdate["Report"],orient='index').T)
        elif tradeupdate['DataType'] == "POSITIONTRACKER":
            #查询持仓监控
            PositionTracker=TCoreAPI.QryPositionTracker()
            #print("持仓监控信息更新\n",PositionTracker)


import numpy as np
import pandas as pd
from nose.tools import assert_equals, assert_is_instance, assert_true

import talib
from talib.test_data import series, assert_np_arrays_equal

def test_MOM():
    values = pd.Series([90.0,88.0,89.0], index=[10, 20, 30])
    result = talib.MOM(values, timeperiod=1)
    assert_is_instance(result, pd.Series)
    assert_np_arrays_equal(result.values, [np.nan, -2, 1])
    assert_np_arrays_equal(result.index, [10, 20, 30])
    result = talib.MOM(values, timeperiod=2)
    assert_is_instance(result, pd.Series)
    assert_np_arrays_equal(result.values, [np.nan, np.nan, -1])
    assert_np_arrays_equal(result.index, [10, 20, 30])
    result = talib.MOM(values, timeperiod=3)
    assert_is_instance(result, pd.Series)
    assert_np_arrays_equal(result.values, [np.nan, np.nan, np.nan])
    assert_np_arrays_equal(result.index, [10, 20, 30])
    result = talib.MOM(values, timeperiod=4)
    assert_is_instance(result, pd.Series)
    assert_np_arrays_equal(result.values, [np.nan, np.nan, np.nan])
    assert_np_arrays_equal(result.index, [10, 20, 30])

def test_MAVP():
    a = pd.Series([1,5,3,4,7,3,8,1,4,6], index=range(10, 20), dtype=float)
    b = pd.Series([2,4,2,4,2,4,2,4,2,4], index=range(20, 30), dtype=float)
    result = talib.MAVP(a, b, minperiod=2, maxperiod=4)
    assert_is_instance(result, pd.Series)
    assert_np_arrays_equal(result.values, [np.nan,np.nan,np.nan,3.25,5.5,4.25,5.5,4.75,2.5,4.75])
    assert_np_arrays_equal(result.index, range(10, 20))
    sma2 = talib.SMA(a, 2)
    assert_is_instance(sma2, pd.Series)
    assert_np_arrays_equal(sma2.index, range(10, 20))
    assert_np_arrays_equal(result.values[4::2], sma2.values[4::2])
    sma4 = talib.SMA(a, 4)
    assert_is_instance(sma4, pd.Series)
    assert_np_arrays_equal(sma4.index, range(10, 20))
    assert_np_arrays_equal(result.values[3::2], sma4.values[3::2])
    result = talib.MAVP(a, b, minperiod=2, maxperiod=3)
    assert_is_instance(result, pd.Series)
    assert_np_arrays_equal(result.values, [np.nan,np.nan,4,4,5.5,4.666666666666667,5.5,4,2.5,3.6666666666666665])
    assert_np_arrays_equal(result.index, range(10, 20))
    sma3 = talib.SMA(a, 3)
    assert_is_instance(sma3, pd.Series)
    assert_np_arrays_equal(sma3.index, range(10, 20))
    assert_np_arrays_equal(result.values[2::2], sma2.values[2::2])
    assert_np_arrays_equal(result.values[3::2], sma3.values[3::2])



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
'''
@File    :   data_process.py
@Time    :   2022/01/14 16:00:06
@Author  :   DingWenjie
@Contact :   359582058@qq.com
@Desc    :   None
'''

import pandas as pd

class BackTest(object):
    '''process etf option data from Wind
    TODO
    '''
    def __init__(self, option_data, etf_data) -> None:
        self.date = list(option_data['date'])
        self.df = option_data
        self.etf = list(etf_data['CLOSE'])


import numpy as np
from nose.tools import assert_equals, assert_true, assert_raises

import talib
from talib import func
from talib.test_data import series, assert_np_arrays_equal, assert_np_arrays_not_equal

def test_talib_version():
    assert_equals(talib.__ta_version__[:5], b'0.4.0')

def test_num_functions():
    assert_equals(len(talib.get_functions()), 158)

def test_input_wrong_type():
    a1 = np.arange(10, dtype=int)
    with assert_raises(Exception):
        func.MOM(a1)

def test_input_lengths():
    a1 = np.arange(10, dtype=float)
    a2 = np.arange(11, dtype=float)
    with assert_raises(Exception):
        func.BOP(a2, a1, a1, a1)
    with assert_raises(Exception):
        func.BOP(a1, a2, a1, a1)
    with assert_raises(Exception):
        func.BOP(a1, a1, a2, a1)
    with assert_raises(Exception):
        func.BOP(a1, a1, a1, a2)

def test_input_nans():
    a1 = np.arange(10, dtype=float)
    a2 = np.arange(10, dtype=float)
    a2[0] = np.nan
    a2[1] = np.nan
    r1, r2 = func.AROON(a1, a2, 2)
    assert_np_arrays_equal(r1, [np.nan, np.nan, np.nan, np.nan, 0, 0, 0, 0, 0, 0])
    assert_np_arrays_equal(r2, [np.nan, np.nan, np.nan, np.nan, 100, 100, 100, 100, 100, 100])
    r1, r2 = func.AROON(a2, a1, 2)
    assert_np_arrays_equal(r1, [np.nan, np.nan, np.nan, np.nan, 0, 0, 0, 0, 0, 0])
    assert_np_arrays_equal(r2, [np.nan, np.nan, np.nan, np.nan, 100, 100, 100, 100, 100, 100])

def test_unstable_period():
    a = np.arange(10, dtype=float)
    r = func.EMA(a, 3)
    assert_np_arrays_equal(r, [np.nan, np.nan, 1, 2, 3, 4, 5, 6, 7, 8])
    talib.set_unstable_period('EMA', 5)
    r = func.EMA(a, 3)
    assert_np_arrays_equal(r, [np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, np.nan, 6, 7, 8])
    talib.set_unstable_period('EMA', 0)

def test_compatibility():
    a = np.arange(10, dtype=float)
    talib.set_compatibility(0)
    r = func.EMA(a, 3)
    assert_np_arrays_equal(r, [np.nan, np.nan, 1, 2, 3, 4, 5, 6, 7, 8])
    talib.set_compatibility(1)
    r = func.EMA(a, 3)
    assert_np_arrays_equal(r, [np.nan, np.nan,1.25,2.125,3.0625,4.03125,5.015625,6.0078125,7.00390625,8.001953125])
    talib.set_compatibility(0)

def test_MIN():
    result = func.MIN(series, timeperiod=4)
    i = np.where(~np.isnan(result))[0][0]
    assert_equals(len(series), len(result))
    assert_equals(result[i + 1], 93.780)
    assert_equals(result[i + 2], 93.780)
    assert_equals(result[i + 3], 92.530)
    assert_equals(result[i + 4], 92.530)
    values = np.array([np.nan, 5., 4., 3., 5., 7.])
    result = func.MIN(values, timeperiod=2)
    assert_np_arrays_equal(result, [np.nan, np.nan, 4, 3, 3, 5])

def test_MAX():
    result = func.MAX(series, timeperiod=4)
    i = np.where(~np.isnan(result))[0][0]
    assert_equals(len(series), len(result))
    assert_equals(result[i + 2], 95.090)
    assert_equals(result[i + 3], 95.090)
    assert_equals(result[i + 4], 94.620)
    assert_equals(result[i + 5], 94.620)

def test_MOM():
    values = np.array([90.0,88.0,89.0])
    result = func.MOM(values, timeperiod=1)
    assert_np_arrays_equal(result, [np.nan, -2, 1])
    result = func.MOM(values, timeperiod=2)
    assert_np_arrays_equal(result, [np.nan, np.nan, -1])
    result = func.MOM(values, timeperiod=3)
    assert_np_arrays_equal(result, [np.nan, np.nan, np.nan])
    result = func.MOM(values, timeperiod=4)
    assert_np_arrays_equal(result, [np.nan, np.nan, np.nan])

def test_BBANDS():
    upper, middle, lower = func.BBANDS(series, timeperiod=20,
                                        nbdevup=2.0, nbdevdn=2.0,
                                        matype=talib.MA_Type.EMA)
    i = np.where(~np.isnan(upper))[0][0]
    assert_true(len(upper) == len(middle) == len(lower) == len(series))
    #assert_true(abs(upper[i + 0] - 98.0734) < 1e-3)
    assert_true(abs(middle[i + 0] - 92.8910) < 1e-3)
    assert_true(abs(lower[i + 0] - 87.7086) < 1e-3)
    #assert_true(abs(upper[i + 13] - 93.674) < 1e-3)
    assert_true(abs(middle[i + 13] - 87.679) < 1e-3)
    assert_true(abs(lower[i + 13] - 81.685) < 1e-3)

def test_DEMA():
    result = func.DEMA(series)
    i = np.where(~np.isnan(result))[0][0]
    assert_true(len(series) == len(result))
    assert_true(abs(result[i + 1] - 86.765) < 1e-3)
    assert_true(abs(result[i + 2] - 86.942) < 1e-3)
    assert_true(abs(result[i + 3] - 87.089) < 1e-3)
    assert_true(abs(result[i + 4] - 87.656) < 1e-3)

def test_EMAEMA():
    result = func.EMA(series, timeperiod=2)
    result = func.EMA(result, timeperiod=2)
    i = np.where(~np.isnan(result))[0][0]
    assert_true(len(series) == len(result))
    assert_equals(i, 2)

def test_CDL3BLACKCROWS():
    o = np.array([39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 39.00, 40.32, 40.51, 38.09, 35.00, 27.66, 30.80])
    h = np.array([40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 40.84, 41.69, 40.84, 38.12, 35.50, 31.74, 32.51])
    l = np.array([35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 35.80, 39.26, 36.73, 33.37, 30.03, 27.03, 28.31])
    c = np.array([40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.29, 40.46, 37.08, 33.37, 30.03, 31.46, 28.31])

    result = func.CDL3BLACKCROWS(o, h, l, c)
    assert_np_arrays_equal(result, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -100, 0, 0])

def test_RSI():
    a = np.array([0.00000024, 0.00000024, 0.00000024,
      0.00000024, 0.00000024, 0.00000023,
      0.00000024, 0.00000024, 0.00000024,
      0.00000024, 0.00000023, 0.00000024,
      0.00000023, 0.00000024, 0.00000023,
      0.00000024, 0.00000024, 0.00000023,
      0.00000023, 0.00000023], dtype='float64')
    result = func.RSI(a, 10)
    assert_np_arrays_equal(result, [np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,0,0,0,0,0,0,0,0,0,0])
    result = func.RSI(a * 100000, 10)
    assert_np_arrays_equal(result, [np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,np.nan,33.333333333333329,51.351351351351347,39.491916859122398,51.84807024709005,42.25953803191981,52.101824405061215,52.101824405061215,43.043664867691085,43.043664867691085,43.043664867691085])

def test_MAVP():
    a = np.array([1,5,3,4,7,3,8,1,4,6], dtype=float)
    b = np.array([2,4,2,4,2,4,2,4,2,4], dtype=float)
    result = func.MAVP(a, b, minperiod=2, maxperiod=4)
    assert_np_arrays_equal(result, [np.nan,np.nan,np.nan,3.25,5.5,4.25,5.5,4.75,2.5,4.75])
    sma2 = func.SMA(a, 2)
    assert_np_arrays_equal(result[4::2], sma2[4::2])
    sma4 = func.SMA(a, 4)
    assert_np_arrays_equal(result[3::2], sma4[3::2])
    result = func.MAVP(a, b, minperiod=2, maxperiod=3)
    assert_np_arrays_equal(result, [np.nan,np.nan,4,4,5.5,4.666666666666667,5.5,4,2.5,3.6666666666666665])
    sma3 = func.SMA(a, 3)
    assert_np_arrays_equal(result[2::2], sma2[2::2])
    assert_np_arrays_equal(result[3::2], sma3[3::2])



import numpy as np
from nose.tools import (
    assert_equals,
    assert_true,
    assert_false,
    assert_raises,
    )

try:
    from collections import OrderedDict
except ImportError: # handle python 2.6 and earlier
    from ordereddict import OrderedDict

import talib
from talib import func
from talib import abstract
from talib.test_data import ford_2012, assert_np_arrays_equal, assert_np_arrays_not_equal


def test_pandas():
    import pandas
    input_df = pandas.DataFrame(ford_2012)
    input_dict = dict((k, pandas.Series(v)) for k, v in ford_2012.items())

    expected_k, expected_d = func.STOCH(ford_2012['high'], ford_2012['low'], ford_2012['close']) # 5, 3, 0, 3, 0
    output = abstract.Function('stoch', input_df).outputs
    assert_true(isinstance(output, pandas.DataFrame))
    assert_np_arrays_equal(expected_k, output['slowk'])
    assert_np_arrays_equal(expected_d, output['slowd'])
    output = abstract.Function('stoch', input_dict).outputs
    assert_true(isinstance(output, list))
    assert_np_arrays_equal(expected_k, output[0])
    assert_np_arrays_equal(expected_d, output[1])

    expected = func.SMA(ford_2012['close'], 10)
    output = abstract.Function('sma', input_df, 10).outputs
    assert_true(isinstance(output, pandas.Series))
    assert_np_arrays_equal(expected, output)
    output = abstract.Function('sma', input_dict, 10).outputs
    assert_true(isinstance(output, np.ndarray))
    assert_np_arrays_equal(expected, output)


def test_pandas_series():
    import pandas
    input_df = pandas.DataFrame(ford_2012)
    output = talib.SMA(input_df['close'], 10)
    expected = pandas.Series(func.SMA(ford_2012['close'], 10),
                             index=input_df.index)
    pandas.util.testing.assert_series_equal(output, expected)

    # Test kwargs
    output = talib.SMA(real=input_df['close'], timeperiod=10)
    pandas.util.testing.assert_series_equal(output, expected)

    # Test talib.func API
    output = func.SMA(input_df['close'], timeperiod=10)
    pandas.util.testing.assert_series_equal(output, expected)

    # Test multiple outputs such as from BBANDS
    _, output, _ = talib.BBANDS(input_df['close'], 10)
    expected = pandas.Series(func.BBANDS(ford_2012['close'], 10)[1],
                             index=input_df.index)
    pandas.util.testing.assert_series_equal(output, expected)


def test_SMA():
    expected = func.SMA(ford_2012['close'], 10)
    assert_np_arrays_equal(expected, abstract.Function('sma', ford_2012, 10).outputs)
    assert_np_arrays_equal(expected, abstract.Function('sma')(ford_2012, 10, price='close'))
    assert_np_arrays_equal(expected, abstract.Function('sma')(ford_2012, timeperiod=10))
    expected = func.SMA(ford_2012['open'], 10)
    assert_np_arrays_equal(expected, abstract.Function('sma', ford_2012, 10, price='open').outputs)
    assert_np_arrays_equal(expected, abstract.Function('sma', price='low')(ford_2012, 10, price='open'))
    assert_np_arrays_not_equal(expected, abstract.Function('sma', ford_2012, 10, price='open')(timeperiod=20))
    assert_np_arrays_not_equal(expected, abstract.Function('sma', ford_2012)(10, price='close'))
    assert_np_arrays_not_equal(expected, abstract.Function('sma', 10)(ford_2012, price='high'))
    assert_np_arrays_not_equal(expected, abstract.Function('sma', price='low')(ford_2012, 10))
    input_arrays = {'foobarbaz': ford_2012['open']}
    assert_np_arrays_equal(expected, abstract.SMA(input_arrays, 10, price='foobarbaz'))


def test_STOCH():
    # check defaults match
    expected_k, expected_d = func.STOCH(ford_2012['high'], ford_2012['low'], ford_2012['close']) # 5, 3, 0, 3, 0
    got_k, got_d = abstract.Function('stoch', ford_2012).outputs
    assert_np_arrays_equal(expected_k, got_k)
    assert_np_arrays_equal(expected_d, got_d)

    expected_k, expected_d = func.STOCH(ford_2012['high'], ford_2012['low'], ford_2012['close'])
    got_k, got_d = abstract.Function('stoch', ford_2012)(5, 3, 0, 3, 0)
    assert_np_arrays_equal(expected_k, got_k)
    assert_np_arrays_equal(expected_d, got_d)

    expected_k, expected_d = func.STOCH(ford_2012['high'], ford_2012['low'], ford_2012['close'], 15)
    got_k, got_d = abstract.Function('stoch', ford_2012)(15, 5, 0, 5, 0)
    assert_np_arrays_not_equal(expected_k, got_k)
    assert_np_arrays_not_equal(expected_d, got_d)

    expected_k, expected_d = func.STOCH(ford_2012['high'], ford_2012['low'], ford_2012['close'], 15, 5, 1, 5, 1)
    got_k, got_d = abstract.Function('stoch', ford_2012)(15, 5, 1, 5, 1)
    assert_np_arrays_equal(expected_k, got_k)
    assert_np_arrays_equal(expected_d, got_d)

def test_doji_candle():
    expected = func.CDLDOJI(ford_2012['open'], ford_2012['high'], ford_2012['low'], ford_2012['close'])
    got = abstract.Function('CDLDOJI').run(ford_2012)
    assert_np_arrays_equal(got, expected)

def test_MAVP():
    mavp = abstract.MAVP
    assert_raises(Exception, mavp.set_input_arrays, ford_2012)
    input_d = {}
    input_d['close'] = ford_2012['close']
    input_d['periods'] = np.arange(30)
    assert_true(mavp.set_input_arrays(input_d))
    assert_equals(mavp.input_arrays, input_d)

def test_info():
    stochrsi = abstract.Function('STOCHRSI')
    stochrsi.input_names = {'price': 'open'}
    stochrsi.parameters = {'fastd_matype': talib.MA_Type.EMA}
    expected = {
        'display_name': 'Stochastic Relative Strength Index',
        'function_flags': ['Function has an unstable period'],
        'group': 'Momentum Indicators',
        'input_names': OrderedDict([('price', 'open')]),
        'name': 'STOCHRSI',
        'output_flags': OrderedDict([
            ('fastk', ['Line']),
            ('fastd', ['Line']),
            ]),
        'output_names': ['fastk', 'fastd'],
        'parameters': OrderedDict([
            ('timeperiod', 14),
            ('fastk_period', 5),
            ('fastd_period', 3),
            ('fastd_matype', 1),
            ]),
        }
    assert_equals(expected, stochrsi.info)

    expected = {
        'display_name': 'Bollinger Bands',
        'function_flags': ['Output scale same as input'],
        'group': 'Overlap Studies',
        'input_names': OrderedDict([('price', 'close')]),
        'name': 'BBANDS',
        'output_flags': OrderedDict([
            ('upperband', ['Values represent an upper limit']),
            ('middleband', ['Line']),
            ('lowerband', ['Values represent a lower limit']),
            ]),
        'output_names': ['upperband', 'middleband', 'lowerband'],
        'parameters': OrderedDict([
            ('timeperiod', 5),
            ('nbdevup', 2),
            ('nbdevdn', 2),
            ('matype', 0),
            ]),
        }
    assert_equals(expected, abstract.Function('BBANDS').info)

def test_input_names():
    expected = OrderedDict([('price', 'close')])
    assert_equals(expected, abstract.Function('MAMA').input_names)

    # test setting input_names
    obv = abstract.Function('OBV')
    expected = OrderedDict([
        ('price', 'open'),
        ('prices', ['volume']),
        ])
    obv.input_names = expected
    assert_equals(obv.input_names, expected)

    obv.input_names = {
        'price': 'open',
        'prices': ['volume'],
        }
    assert_equals(obv.input_names, expected)

def test_input_arrays():
    mama = abstract.Function('MAMA')
    # test default setting
    expected = {
        'open': None,
        'high': None,
        'low': None,
        'close': None,
        'volume': None,
        }
    assert_equals(expected, mama.get_input_arrays())
    # test setting/getting input_arrays
    assert_true(mama.set_input_arrays(ford_2012))
    assert_equals(mama.get_input_arrays(), ford_2012)
    assert_raises(Exception,
                  mama.set_input_arrays, {'hello': 'fail', 'world': 'bye'})

    # test only required keys are needed
    willr = abstract.Function('WILLR')
    reqd = willr.input_names['prices']
    input_d = dict([(key, ford_2012[key]) for key in reqd])
    assert_true(willr.set_input_arrays(input_d))
    assert_equals(willr.input_arrays, input_d)

    # test extraneous keys are ignored
    input_d['extra_stuffs'] = 'you should never see me'
    input_d['date'] = np.random.rand(100)
    assert_true(willr.set_input_arrays(input_d))

    # test missing keys get detected
    input_d['open'] = ford_2012['open']
    input_d.pop('close')
    assert_raises(Exception, willr.set_input_arrays, input_d)

    # test changing input_names on the Function
    willr.input_names = {'prices': ['high', 'low', 'open']}
    assert_true(willr.set_input_arrays(input_d))

def test_parameters():
    stoch = abstract.Function('STOCH')
    expected = OrderedDict([
        ('fastk_period', 5),
        ('slowk_period', 3),
        ('slowk_matype', 0),
        ('slowd_period', 3),
        ('slowd_matype', 0),
        ])
    assert_equals(expected, stoch.parameters)

    stoch.parameters = {'fastk_period': 10}
    expected['fastk_period'] = 10
    assert_equals(expected, stoch.parameters)

    stoch.parameters = {'slowk_period': 8, 'slowd_period': 5}
    expected['slowk_period'] = 8
    expected['slowd_period'] = 5
    assert_equals(expected, stoch.parameters)

    stoch.parameters = {'slowd_matype': talib.MA_Type.T3}
    expected['slowd_matype'] = 8
    assert_equals(expected, stoch.parameters)

    stoch.parameters = {
        'slowk_matype': talib.MA_Type.WMA,
        'slowd_matype': talib.MA_Type.EMA,
        }
    expected['slowk_matype'] = 2
    expected['slowd_matype'] = 1
    assert_equals(expected, stoch.parameters)

def test_lookback():
    assert_equals(abstract.Function('SMA', 10).lookback, 9)

    stochrsi = abstract.Function('stochrsi', 20, 5, 3)
    assert_equals(stochrsi.lookback, 26)

def test_call_supports_same_signature_as_func_module():
    adx = abstract.Function('ADX')

    expected = func.ADX(ford_2012['open'], ford_2012['high'], ford_2012['low'])
    output = adx(ford_2012['open'], ford_2012['high'], ford_2012['low'])
    assert_np_arrays_equal(expected, output)

    with assert_raises(TypeError) as e:
        adx(ford_2012['open'], ford_2012['high'], ford_2012['low'], ford_2012['close'])
        assert 'Too many price arguments: expected 3 (open, high, low)' in str(
            e.exception.message)

    with assert_raises(TypeError) as e:
        adx(ford_2012['open'], ford_2012['high'])
        assert 'Not enough price arguments: expected 3 (open, high, low)' in str(
            e.exception.message)

def test_parameter_type_checking():
    sma = abstract.Function('SMA', timeperiod=10)

    expected_error = 'Invalid parameter value for timeperiod (expected int, got float)'

    with assert_raises(TypeError) as e:
        sma(ford_2012['close'], 35.5)
        assert expected_error in str(e.exception.message)

    with assert_raises(TypeError) as e:
        abstract.Function('SMA', timeperiod=35.5)
        assert expected_error in str(e.exception.message)

    with assert_raises(TypeError) as e:
        sma.parameters = {'timeperiod': 35.5}
        assert expected_error in str(e.exception.message)

    with assert_raises(TypeError) as e:
        sma.set_parameters(timeperiod=35.5)
        assert expected_error in str(e.exception.message)

def test_call_doesnt_cache_parameters():
    sma = abstract.Function('SMA', timeperiod=10)

    expected = func.SMA(ford_2012['open'], 20)
    output = sma(ford_2012, timeperiod=20, price='open')
    assert_np_arrays_equal(expected, output)

    expected = func.SMA(ford_2012['close'], 20)
    output = sma(ford_2012, timeperiod=20)
    assert_np_arrays_equal(expected, output)

    expected = func.SMA(ford_2012['close'], 10)
    output = sma(ford_2012)
    assert_np_arrays_equal(expected, output)

def test_call_without_arguments():
    with assert_raises(TypeError) as e:
        abstract.Function('SMA')()
        assert 'Not enough price arguments' in str(e.exception.message)

    with assert_raises(TypeError) as e:
        abstract.Function('SMA')(10)
        assert 'Not enough price arguments' in str(e.exception.message)



import talib._ta_lib as _ta_lib
from ._ta_lib import Function as _Function, __TA_FUNCTION_NAMES__, _get_defaults_and_docs

# add some backwards compat for backtrader
from ._ta_lib import TA_FUNC_FLAGS, TA_INPUT_FLAGS, TA_OUTPUT_FLAGS

_func_obj_mapping = {
    func_name: getattr(_ta_lib, func_name)
    for func_name in __TA_FUNCTION_NAMES__
}


def Function(function_name, *args, **kwargs):
    func_name = function_name.upper()
    if func_name not in _func_obj_mapping:
        raise Exception('%s not supported by TA-LIB.' % func_name)

    return _Function(
        func_name, _func_obj_mapping[func_name], *args, **kwargs
    )


for func_name in __TA_FUNCTION_NAMES__:
    globals()[func_name] = Function(func_name)


__all__ = ["Function", "_get_defaults_and_docs"] + __TA_FUNCTION_NAMES__



import time
from tcorepyapi.tcoreapi_mq import *
import pandas as pd
#登入
TCoreAPI= TCoreZMQ(quote_port="51630",trade_port="51600")
#TCoreAPI= TCoreZMQ(quote_port="51864",trade_port="51834")
#TCoreAPI= TCoreZMQ(quote_port="51424",trade_port="51394")
#查询指定合约信息
print("查询指定合约：",TCoreAPI.QueryInstrumentInfo("TC.O.SSE.510050.202108.C.3.6"))

#查询指定类型合约列表
#参数1：
    #期货：Future
    #期权：Options
    #证券：Stock
#参数2：指定日期的合约列表，如果不带或带入空参数，返回最新的合约列表，否则返回指定日期的合约列表
symbollist=TCoreAPI.QueryAllInstrumentInfo("Options")
#print("查询合约：\n",symbollist)
#解析期权合约列表
if symbollist['Success']=="OK":
    for opt in symbollist['Instruments']['Node']:
        #SSE交易所的期权
        if opt['ENG']=='SSE(O)':
            for optlist in opt['Node']:
                #标的50etf的期权
                if optlist['ENG_M']=='50ETF':
                    for optlist2 in optlist['Node']:
                        #202108月份合约
                        if optlist2['ENG']=='202108':
                            for syblist in optlist2['Node']:
                                for symb in syblist['Contracts']:
                                    print(symb)
#解除订阅
#TCoreAPI.UnsubQuote("TC.F.CFFEX.IF.202106")
#订阅实时行情
TCoreAPI.SubQuote("TC.O.SSE.510050.202108.C.3.2")
#解除订阅
#TCoreAPI.UnsubGreeks("TC.F.U_SSE.510050.202106")
#订阅实时Greeks行情
#TCoreAPI.SubGreeks("TC.F.U_SSE.510050.202106")

#获取历史数据
    #1：合约代码，
    #2：数据周期:
        ##标准行情历史数据
        # tick: "TICKS", 
        # 分K: "1K",
        # 日K: "DK"，
        ##DOGS为包含Greeks、标的期货、标的合约的历史数据
        # DOGS秒："DOGSS", 
        # DOGS分K："DOGSK"
    #3: 历史数据开始时间,
    #4: 历史数据结束时间
#his=TCoreAPI.SubHistory("TC.F.U_SSE.510050.202106", "DOGSK", "2021050100", "2021052107")
his=TCoreAPI.SubHistory("TC.O.SSE.510050.202108.C.3.6", "1K", "2021062300", "2021062307")
print("历史数据：\n",his)

#查询已登入资金账户
accountInfo = TCoreAPI.QryAccount()
#查询当日委托回报
reportData = TCoreAPI.QryReport()
if reportData:
    print("当日委托回报：\n",pd.DataFrame(reportData))
else:
    print("今日没有委托记录")
#查询当日成交回报
fillReportData = TCoreAPI.QryFillReport()
if fillReportData:
    print("当日成交回报：\n",pd.DataFrame(fillReportData))
else:
    print("今日没有成交记录")
#查询持仓监控
PositionTracker=TCoreAPI.QryPositionTracker()
if PositionTracker:
    print("持仓监控信息：\n",PositionTracker)
else:
    print("没有持仓监控信息")
strAccountMask =""
if accountInfo != None:
    arrInfo = accountInfo["Accounts"]
    if len(arrInfo) != 0:
        for acc in arrInfo:
            print("当前已登入资金账户\n",pd.DataFrame.from_dict(acc,orient='index').T)
        #获取账户列表中的第一个账户
        strAccountMask = arrInfo[0]["AccountMask"]
        #查询资金
        margin=TCoreAPI.QryMargin(strAccountMask)
        print("资金信息：\n",pd.DataFrame(margin["Margins"]))
        #查询持仓
        pos = TCoreAPI.QryPosition(strAccountMask)
        print("持仓信息：\n",pd.DataFrame(pos))

def mdupdatethread():
    flag=True
    while True:
        #行情更新
        quoteupdate= TCoreAPI.mdupdate()
        if quoteupdate:
            if quoteupdate['DataType']=='REALTIME':
                print("实时行情: \n",pd.DataFrame.from_dict(quoteupdate['Quote'],orient='index').T)  #这里只是展示用，实际应用建议不要使用dataframe转换实时数据，会影响运行效率
                if strAccountMask !="" and flag:
                    #新增一笔委托
                    orders_obj = {
                                    "Symbol":quoteupdate['Quote']['Symbol'],
                                    "BrokerID":arrInfo[0]['BrokerID'],
                                    "Account":arrInfo[0]['Account'],
                                    "Price":"0.0001",#quoteupdate['Quote']['LowPrice'],
                                    "TimeInForce":"1",
                                    "Side":"1",
                                    "OrderType":"2",
                                    "OrderQty":"1",
                                    "PositionEffect":"4",
                                    "SelfTradePrevention":"3"
                                    #"ChasePrice":"1T|5|3|M"
                                }
                    ordid = TCoreAPI.NewOrder(orders_obj)
                    if ordid!=None:
                        while True:
                            if TCoreAPI.getorderinfo(ordid):
                                print("新增委托",TCoreAPI.getorderinfo(ordid)['ReportID'])#,TCoreAPI.QryReport()[-1]['ReportID'])
                                break

                    #改单
                    time.sleep(1)
                    print(TCoreAPI.getorderinfo(ordid)['ExecType'])
                    reporders_obj={
                        "ReportID":TCoreAPI.getorderinfo(ordid)['ReportID'],#TCoreAPI.QryReport()[-1]['ReportID'],
                        "ReplaceExecType":"0",
                        "Price":quoteupdate['Quote']['TradingPrice']
                        }
                    #在可改单的委托单状态下发送改单指令（注意：模拟交易和外盘允许改单，内盘建议使用先删单单然后发送新委托方式改单）
                    if TCoreAPI.getorderinfo(ordid)['ExecType'] in ["0","1","4","6"]:
                        reorder=TCoreAPI.ReplaceOrder(reporders_obj)
                        print("修改委托单：\n",reorder,TCoreAPI.QryReport()[-1]['ReportID'])
                    #取消委托单
                    time.sleep(1)
                    cancalorders_obj = {
                                "ReportID":TCoreAPI.getorderinfo(ordid)['ReportID'],#TCoreAPI.QryReport()[-1]['ReportID'],
                                }
                    #在可删单的委托单状态下发送删单指令      
                    if TCoreAPI.getorderinfo(ordid)['ExecType'] in ["0","1","4","6"]:
                        cancal_order = TCoreAPI.CancelOrder(cancalorders_obj )
                        print("删单:",cancal_order)
                    #建组合
                    orders_obj = {
                                "Symbol":"TC.O.SSE.510050.202106.C.3.6&TC.O.SSE.510050.202106.C.3.7",
                                "BrokerID":arrInfo[0]['BrokerID'],
                                "Account":arrInfo[0]['Account'],
                                "CombDirection":"1",
                                "Side":"1",
                                "CombinationType":"1",
                                "Volume":"1"
                                }
                    TCoreAPI.OptComb(orders_obj)
                    #查询新建组合的委托回报
                    TCoreAPI.QryOptCombOrder(strAccountMask,"")
                    flag=False
            elif  quoteupdate['DataType']=="GREEKS":
                print("实时GREEKS: \n",pd.DataFrame.from_dict(quoteupdate['Quote'],orient='index').T)

def tdupdatethread():
    while True:
        #交易更新
        tradeupdate= TCoreAPI.tdupdate()
        if tradeupdate:
            if tradeupdate['DataType']=='ACCOUNTS':
                print("账户列表更新: \n",tradeupdate['Accounts'])
                arrInfo = tradeupdate['Accounts']
                if len(arrInfo) != 0:
                    strAccountMask = arrInfo[0]["AccountMask"]
            elif tradeupdate['DataType'] == "EXECUTIONREPORT":
                print("实时委托回报: \n",pd.DataFrame.from_dict(tradeupdate["Report"],orient='index').T)
            elif tradeupdate['DataType'] == "FILLEDREPORT":
                print("实时成交: \n",pd.DataFrame.from_dict(tradeupdate["Report"],orient='index').T)
            elif tradeupdate['DataType'] == "POSITIONTRACKER":
                #查询持仓监控
                PositionTracker=TCoreAPI.QryPositionTracker()
                print("持仓监控信息更新\n",PositionTracker)

thr1=threading.Thread(target=mdupdatethread)
thr2=threading.Thread(target=tdupdatethread)
thr2.start()
thr1.start()




# -*- coding: utf-8 -*-
"""
containing the following funcs
==========
    bs_call(S,X,T,rf,sigma)
    bs_put(S,X,T,rf,sigma)
    binomial_grid(n)
    CND(X)
    a
==========
"""

def a(k=1):
    b = 2
    return b
    


def bs_call(S,X,T,rf,sigma):
    """
       Objective: Black-Schole-Merton option model
       Format   : bs_call(S,X,T,r,sigma)
               S: current stock price
               X: exercise price
               T: maturity date in years
              rf: risk-free rate (continusouly compounded)
           sigma: volatiity of underlying security
       Example 1:  
         >>>bs_call(40,40,1,0.1,0.2)
         5.3078706338643578
    """    
    from scipy import log,exp,sqrt,special
    d1=(log(S/X)+(rf+sigma*sigma/2.0)*T)/(sigma*sqrt(T))
    d2 = d1-sigma*sqrt(T)
    return S*special.ndtr(d1)-X*exp(-rf*T)*special.ndtr(d2)


def bs_put(S,X,T,rf,sigma):
    """
       Objective: Black-Schole-Merton option model
       Format   : bs_call(S,X,T,r,sigma)
               S: current stock price
               X: exercise price
               T: maturity date in years
              rf: risk-free rate (continusouly compounded)
           sigma: volatiity of underlying security
       Example 1:
       >>> put=bs_put(40,40,0.5,0.05,0.2)
       >>> round(put,2)
       1.77
    """    
    from scipy import log,exp,sqrt,special
    d1=(log(S/X)+(rf+sigma*sigma/2.)*T)/(sigma*sqrt(T))
    d2 = d1-sigma*sqrt(T)
    return X*exp(-rf*T)*special.ndtr(-d2)-S*special.ndtr(-d1)

def binomial_grid(n):
    import networkx as nx
    #import matplotlib.pyplot as plt
    G=nx.Graph()
    for i in range(0,n+1):    
        for j in range(1,i+2):        
            if i<n:            
                G.add_edge((i,j),(i+1,j))
                G.add_edge((i,j),(i+1,j+1))
    posG={}    #dictionary with nodes position
    for node in G.nodes():    
        posG[node]=(node[0],n+2+node[0]-2*node[1])
    nx.draw(G,pos=posG)      

#from math import sqrt, log, pi,exp
#import re
#--------------------------------------------------------#
#--- Cumulative normal distribution        --------------#
#--------------------------------------------------------#
def CND(X):
    """ Cumulative standard normal distribution
            CND(x): x is a scale
            e.g.,
            >>> CND(0)
            0.5000000005248086
    """
    from scipy import exp,sqrt,pi
    (a1,a2,a3,a4,a5)=(0.31938153,-0.356563782,1.781477937,-1.821255978,1.330274429)
    L = abs(X)
    K = 1.0 / (1.0 + 0.2316419 * L)
    w = 1.0 - 1.0 / sqrt(2*pi)*exp(-L*L/2.) * (a1*K + a2*K*K + a3*pow(K,3) +
    a4*pow(K,4) + a5*pow(K,5))
    if X<0:
        w = 1.0-w
    return w



'''
containing the following funcs
=========
    CalIVCall(S,K,T,r,c)
    CalIVPut(S,K,T,r,p)
    CalDeltaCall(S,K,T,r,iv)
    CalDeltaPut(S,K,T,r,iv)
    CalGammaCall(S,K,T,r,iv)
     ?CalGammaCallPct(S,K,T,r,iv)
    CalGammaPut(S,K,T,r,iv)
     ?CalGammaPutPct(S,K,T,r,iv)
    CalThetaCall(S,K,T,r,iv)
    CalThetaPut(S,K,T,r,iv)
     ?CalThetaCall2(S,K,T,r,iv)
     ?CalThetaPut2(S,K,T,r,iv)
    CalVegaCall(S,K,T,r,iv)
    CalVegaPut(S,K,T,r,iv)
    CalStraddle(S,K,T,r,iv)
    CalStraddle2(TC,TP)
    FindDelta(want_greeks_num,df,greeks='Delta')
    FindDeltaV1(want_greeks_num,df,greeks='Delta')
    FindDeltaV3(want_greeks_num,df,greeks='DELTA')
    CalSkew(call_or_put,df_tmp,greeks='Delta')
    CalSkewV2(call_or_put,df_tmp,greeks='Delta')
    CalSkewV3(call_or_put,df_tmp,greeks='Delta')
    CalSkewV4(call_or_put,df_tmp,greeks='Delta')
    CalSkewV5(call_or_put,df_tmp,greeks='DELTA')
=========  
'''

def CalIVCall(S,K,T,r,c):
   #S for stock price
   #K for strike price
    preciselevel=0.00001 #原来的精度是0.0000001
    
    
    import p4f
    i=1
   
    sigma1=0.0001
    sigma2=2
    sigma=sigma1
   
    while i<1000:
          i+=1
          diff=c-p4f.bs_call(S,K,T,r,sigma)#bs_call(S,X,T,rf,sigma):
         
          if diff>0:
              sigma1=sigma
              sigma=(sigma1+sigma2)/2
             
          else:
              sigma2=sigma
              sigma=(sigma1+sigma2)/2
          if abs(diff)<preciselevel: break
          if sigma2 == sigma1: break
     
    return(sigma)
   
def CalIVPut(S,K,T,r,p):
   
    preciselevel=0.00001
   
    import p4f
    i=1
   
    sigma1=0.0001
    sigma2=4
    sigma=sigma1
   
    while i<1000:
          i+=1
          diff=p-p4f.bs_put(S,K,T,r,sigma)
         
          if diff>0:
              sigma1=sigma
              sigma=(sigma1+sigma2)/2
             
          else:
              sigma2=sigma
              sigma=(sigma1+sigma2)/2
          if abs(diff)<preciselevel: break
          if sigma2 == sigma1: break
     
    return(sigma)
   
def CalDeltaCall(S,K,T,r,iv):
   
    change=0.0000001
    import p4f
   
    price1=p4f.bs_call(S+change,K,T,r,iv)
    price2=p4f.bs_call(S-change,K,T,r,iv)
   
    delta=(price1-price2)/(2*change)
     
    return(delta)

def CalDeltaPut(S,K,T,r,iv):
   
    change=0.0000001
    import p4f
   
    price1=p4f.bs_put(S+change,K,T,r,iv)
    price2=p4f.bs_put(S-change,K,T,r,iv)
    delta=(price1-price2)/(2*change)
     
    return(delta)

def CalGammaCall(S,K,T,r,iv):
    ds=0.00001
    orig_delta = CalDeltaCall(S,K,T,r,iv)
    after_delta = CalDeltaCall(S+ds,K,T,r,iv)
    gamma = (after_delta - orig_delta) / ds
    return(gamma)



def CalGammaCallPct(S,K,T,r,iv):
    ds=0.005
    orig_delta = CalDeltaCall(S*(1-ds),K,T,r,iv)
    after_delta = CalDeltaCall(S*(1+ds),K,T,r,iv)
    gamma = (after_delta - orig_delta) 
    return(gamma)


def CalGammaPut(S,K,T,r,iv):
    ds=0.00001
    orig_delta = CalDeltaPut(S,K,T,r,iv)
    after_delta = CalDeltaPut(S+ds,K,T,r,iv)
    gamma = (after_delta - orig_delta) / ds
    return(gamma)
    
def CalGammaPutPct(S,K,T,r,iv):
    ds=0.005
    orig_delta = CalDeltaPut(S*(1-ds),K,T,r,iv)
    after_delta = CalDeltaPut(S*(1+ds),K,T,r,iv)
    gamma = (after_delta - orig_delta) 
    return(gamma)
#############################################################
def CalSpeedCall(S,K,T,r,iv):
    ds=0.00001
    orig_gamma = CalGammaCall(S,K,T,r,iv)
    after_gamma = CalGammaCall(S+ds,K,T,r,iv)
    speed = (after_gamma - orig_gamma) / ds
    return(speed)
    
def CalSpeedCallPct(S,K,T,r,iv):
    ds=0.005
    orig_gamma = CalGammaCallPct(S*(1-ds),K,T,r,iv)
    after_gamma = CalGammaCallPct(S*(1+ds),K,T,r,iv)
    speed = (after_gamma - orig_gamma)
    return(speed)
    
def CalSpeedPutPct(S,K,T,r,iv):
    ds=0.005
    orig_gamma = CalGammaPutPct(S*(1-ds),K,T,r,iv)
    after_gamma = CalGammaPutPct(S*(1+ds),K,T,r,iv)
    speed = (after_gamma - orig_gamma)
    return(speed)

def CalVannaCallPct(S,K,T,r,iv):
    ds=0.005
    orig_vega = CalVegaCall(S*(1-ds),K,T,r,iv)
    after_vega = CalVegaCall(S*(1+ds),K,T,r,iv)
    vanna = (after_vega - orig_vega)
    return(vanna)

def CalVannaPutPct(S,K,T,r,iv):
    ds=0.005
    orig_vega = CalVegaPut(S*(1-ds),K,T,r,iv)
    after_vega = CalVegaPut(S*(1+ds),K,T,r,iv)
    vanna = (after_vega - orig_vega)
    return(vanna)

def CalZommaCallPct(S,K,T,r,iv):
    ds=0.005
    orig_vanna = CalVannaCallPct(S*(1-ds),K,T,r,iv)
    after_vanna = CalVannaCallPct(S*(1+ds),K,T,r,iv)
    zomma = (after_vanna - orig_vanna)
    return(zomma)

def CalZommaPutPct(S,K,T,r,iv):
    ds=0.005
    orig_vanna = CalVannaPutPct(S*(1-ds),K,T,r,iv)
    after_vanna = CalVannaPutPct(S*(1+ds),K,T,r,iv)
    zomma = (after_vanna - orig_vanna)
    return(zomma)








def CalSpeedPut(S,K,T,r,iv):
    ds=0.00001
    orig_gamma = CalGammaPut(S,K,T,r,iv)
    after_gamma = CalGammaPut(S+ds,K,T,r,iv)
    speed = (after_gamma - orig_gamma) / ds
    return(speed)

#########################################################################
def CalThetaCall(S,K,T,r,iv):
    import p4f
    num = 244
    dt = 0.00001
    if T>dt:
        price1=p4f.bs_call(S,K,T,r,iv)
        price2=p4f.bs_call(S,K,T-dt,r,iv)
        theta=(price2-price1) / (dt * num)
    else:
        theta=0
    return(theta)

def CalThetaPut(S,K,T,r,iv):
    import p4f
    num = 244
    dt = 0.00001
    if T>dt:
        price1=p4f.bs_put(S,K,T,r,iv)
        price2=p4f.bs_put(S,K,T-dt,r,iv)
        theta=(price2-price1) / (dt * num)
    else:
        theta=0
    return(theta)
    
def CalThetaCall2(S,K,T,r,iv):
    import p4f
    #file_path='D:\\MyPython\\AnnualTradingDayNum'
    #f = open(file_path,"r")   #设置文件对象
    #ATDN= str(f.readlines())  #直接将文件中按行读到list里，效果与方法2一样
    #f.close()  
    #AnnualTradingDayNum=float(ATDN[2:-2])
    #dt=1/AnnualTradingDayNum
    num = 244
    dt = 1/num
    if T>dt:
        price1=p4f.bs_call(S,K,T,r,iv)
        price2=p4f.bs_call(S,K,T-dt,r,iv)
        theta=(price2-price1)
    else:
        theta=0
    return(theta)

def CalThetaPut2(S,K,T,r,iv):
    import p4f
    #file_path='D:\\MyPthon\\AnnualTradingDayNum'
    #f = open(file_path,"r")   #设置文件对象
    #ATDN= str(f.readlines())  #直接将文件中按行读到list里，效果与方法2一样
    #f.close()  
    #AnnualTradingDayNum=float(ATDN[2:-2])
    #dt=1/AnnualTradingDayNum
    num = 244
    dt = 1/num
    if T>dt:
        price1=p4f.bs_put(S,K,T,r,iv)
        price2=p4f.bs_put(S,K,T-dt,r,iv)
        theta=(price2-price1)
    else:
        theta=0
    return(theta)

def CalVegaCall(S,K,T,r,iv):
    import p4f
    dv=0.005
    if float(iv)>dv:
        price1=p4f.bs_call(S,K,T,r,iv+dv)
        price2=p4f.bs_call(S,K,T,r,iv-dv)
    else:
        price1=p4f.bs_put(S,K,T,r,iv+2*dv)
        price2=p4f.bs_put(S,K,T,r,iv)
        
    vega=price1-price2
    
    return(vega)

def CalVegaPut(S,K,T,r,iv):
    import p4f
    dv=0.005
    if iv>dv:
        price1=p4f.bs_put(S,K,T,r,iv+dv)
        price2=p4f.bs_put(S,K,T,r,iv-dv)
    else:
        price1=p4f.bs_put(S,K,T,r,iv+2*dv)
        price2=p4f.bs_put(S,K,T,r,iv)
    vega=price1-price2
    return(vega)


def CalVannaCall(S,K,T,r,iv):
    dv=0.005
    import p4f
    if float(iv)>dv:
        delta1 = CalDeltaCall(S,K,T,r,iv+dv)
        delta2 = CalDeltaCall(S,K,T,r,iv-dv)
    else:
        delta1 = CalDeltaCall(S,K,T,r,iv+2*dv)
        delta2 = CalDeltaCall(S,K,T,r,iv)
    
    vanna = delta1-delta2
    return(vanna)
    '''
    ds=0.01
    dv=0.005
    import p4f
    if float(iv)>dv:
        price1=p4f.bs_call(S+ds,K,T,r,iv+dv)
        price2=p4f.bs_call(S-ds,K,T,r,iv-dv)
    else:
        price1=p4f.bs_call(S+2*ds,K,T,r,iv+2*dv)
        price2=p4f.bs_call(S,K,T,r,iv)
# !!!!
    vanna=(price1-price2)/(2*ds)
    return(vanna)
    '''

    

def CalVannaPut(S,K,T,r,iv):
    dv=0.005
    import p4f
    if float(iv)>dv:
        delta1 = CalDeltaPut(S,K,T,r,iv+dv)
        delta2 = CalDeltaPut(S,K,T,r,iv-dv)
    else:
        delta1 = CalDeltaPut(S,K,T,r,iv+2*dv)
        delta2 = CalDeltaPut(S,K,T,r,iv)
    
    vanna = delta1-delta2
    return(vanna)
    
    '''
    ds=0.01
    dv=0.005
    import p4f
    if float(iv)>dv:
        price1=p4f.bs_put(S+ds,K,T,r,iv+dv)
        price2=p4f.bs_put(S-ds,K,T,r,iv-dv)
    else:
        price1=p4f.bs_put(S+2*ds,K,T,r,iv+2*dv)
        price2=p4f.bs_put(S,K,T,r,iv)
        
    vanna=(price1-price2)/(2*ds)
    return(vanna)
    '''
####################################
def CalZommaCall(S,K,T,r,iv):
    ds=0.00001
    orig_vanna = CalVannaCall(S,K,T,r,iv)
    after_vanna = CalVannaCall(S+ds,K,T,r,iv)
    zomma = (after_vanna - orig_vanna) / ds
    return(zomma)

def CalZommaPut(S,K,T,r,iv):
    ds=0.00001
    orig_vanna = CalVannaPut(S,K,T,r,iv)
    after_vanna = CalVannaPut(S+ds,K,T,r,iv)
    zomma = (after_vanna - orig_vanna) / ds
    return(zomma)

#####################################################################################



def CalCharmCall(S,K,T,r,iv):
    #charm 隔日delta值的变化
    import p4f
    num = 244
    dt=1/num
    if T>dt:
        delta1 = CalDeltaCall(S,K,T,r,iv)
        delta2 = CalDeltaCall(S,K,T-dt,r,iv)
        charm = delta2-delta1
    else:
        charm=0
    return(charm)
    '''
    import p4f
    ds=0.01
    num = 244
    dt=1/num
    if T>dt:
        price1=p4f.bs_call(S+ds,K,T,r,iv)
        price2=p4f.bs_call(S-ds,K,T-dt,r,iv)
        charm=(price2-price1)/(2*ds)
    else:
        charm=0
    return(charm)
    '''


def CalCharmPut(S,K,T,r,iv):
    import p4f
    num = 244
    dt=1/num
    if T>dt:
        delta1 = CalDeltaPut(S,K,T,r,iv)
        delta2 = CalDeltaPut(S,K,T-dt,r,iv)
        charm = delta2-delta1
    else:
        charm=0
    return(charm)
    
    '''
    import p4f
    ds=0.01
    num = 244
    dt=1/num
    if T>dt:
        price1=p4f.bs_put(S+ds,K,T,r,iv)
        price2=p4f.bs_put(S-ds,K,T-dt,r,iv)
        charm=(price2-price1)/(2*ds)
    else:
        charm=0
    return(charm)
    '''


def CalVommaCall(S,K,T,r,iv):
    import p4f
    dv=0.005  
    if float(iv)>dv:
        ori_vega = CalVegaCall(S,K,T,r,iv-dv)
        after_vega = CalVegaCall(S,K,T,r,iv+dv)
    else:
        ori_vega = CalVegaCall(S,K,T,r,iv)
        after_vega = CalVegaCall(S,K,T,r,iv+2*dv)
    vomma = after_vega - ori_vega
    return(vomma)

def CalVommaPut(S,K,T,r,iv):
    import p4f
    dv=0.005  
    if float(iv)>dv:
        ori_vega = CalVegaPut(S,K,T,r,iv-dv)
        after_vega = CalVegaPut(S,K,T,r,iv+dv)
    else:
        ori_vega = CalVegaPut(S,K,T,r,iv)
        after_vega = CalVegaPut(S,K,T,r,iv+2*dv)
    vomma = after_vega - ori_vega
    return(vomma)



def CalCallPremium(S,K,T,r,iv):
    import p4f
    premium=p4f.bs_call(S,K,T,r,iv)
    return(premium)

def CalPutPremium(S,K,T,r,iv):
    import p4f
    premium=p4f.bs_put(S,K,T,r,iv)
    return(premium)
    
# staddle 
def CalStraddle(S,K,T,r,iv):
    CallATM=CalCallPremium(S,K,T,r,iv)
    PutATM=CalPutPremium(S,K,T,r,iv)
    Straddle=CallATM+PutATM
    return (Straddle)

def CalStraddle2(TC,TP):  #这里的关键是delta=0.5的这个期权是正好对应平值期权
    TC['DeltaDiff']=abs(TC['Delta']-0.5)  
    index1=TC['DeltaDiff'].idxmin()
    TC['DeltaDiff'].iloc[index1]=100000.0
    index2=TC['DeltaDiff'].idxmin()
    x0=TC['Delta'].iloc[index1]  
    x1=TC['Delta'].iloc[index2]
    y0=TC['Price'].iloc[index1]
    y1=TC['Price'].iloc[index2]
    CallATM=(0.5-x1)*(y0-y1)/(x0-x1)+y1
    
    TP['DeltaDiff']=abs(TP['Delta']+0.5)  
    index1=TP['DeltaDiff'].idxmin()
    TP['DeltaDiff'].iloc[index1]=100000.0
    index2=TP['DeltaDiff'].idxmin()
    x0=TP['Delta'].iloc[index1]  
    x1=TP['Delta'].iloc[index2]
    y0=TP['Price'].iloc[index1]
    y1=TP['Price'].iloc[index2]
    PutATM=(-0.5-x1)*(y0-y1)/(x0-x1)+y1
    Straddle=CallATM+PutATM
    
    return (Straddle)
    

    
    
def FindDelta(want_greeks_num,df,greeks='Delta'):
    import pandas as pd
    import copy
    df_new=[]
    tmp = copy.deepcopy(df)
    tmp[greeks] = df[greeks].apply(lambda row: abs(row-want_greeks_num))
    df_new.append(df[tmp[greeks] == tmp[greeks].min()])
    tmp.loc[tmp[greeks] == tmp[greeks].min(),greeks] = 1000
    df_new.append(df[tmp[greeks] == tmp[greeks].min()])
    df_new2 = pd.concat([df_new[0],df_new[1]],ignore_index= False)    
    df_new2 = df_new2.reset_index(drop = True)
    return(df_new2)
def FindDeltaV1(want_greeks_num,df,greeks='Delta'):
    import pandas as pd
    import copy
    df_new=[]
    tmp = copy.deepcopy(df)
    tmp[greeks] = df[greeks].apply(lambda row: abs(row-want_greeks_num))
    df_new.append(df[tmp[greeks] == tmp[greeks].min()])
    tmp.loc[tmp[greeks] == tmp[greeks].min(),greeks] = 1000
    df_new.append(df[tmp[greeks] == tmp[greeks].min()])
    df_new2 = pd.concat([df_new[0],df_new[1]],ignore_index= False)    
    df_new2 = df_new2.reset_index(drop = True)
    return(df_new2)
    
def FindDeltaV3(want_greeks_num,df,greeks='DELTA'):
    import pandas as pd
    import copy
    df_new=[]
#    EmptyFlag=0
    tmp = copy.deepcopy(df)
    tmp['DelAbs']=0
    tmp[greeks] = df[greeks]-want_greeks_num
    if  tmp[tmp[greeks]>0].empty or tmp[tmp[greeks]<0].empty:
#        EmptyFlag=1
        tmp['DelAbs'] = abs(df[greeks]-want_greeks_num)
#        df1=df[tmp['DelAbs'] == tmp['DelAbs'].min()]
#        tmp=df1.reset_index(drop=True)
        df_new.append(df[tmp['DelAbs'] == tmp['DelAbs'].min()])
        tmp.loc[tmp['DelAbs'] == tmp['DelAbs'].min(),'DelAbs'] = 1000
        df_new.append(df[tmp['DelAbs'] == tmp['DelAbs'].min()])
        df_new2 = pd.concat([df_new[0],df_new[1]],ignore_index= False)    
        df_new2 = df_new2.reset_index(drop = True)
    else:
        df_new.append(df[tmp[greeks] == tmp[tmp[greeks]>0][greeks].min()])
        df_new.append(df[tmp[greeks] == tmp[tmp[greeks]<0][greeks].max()])
    df_new2 = pd.concat([df_new[0],df_new[1]],ignore_index= False)    
    df_new2 = df_new2.reset_index(drop = True)
#    print(df_new2)
    return(df_new2)

def CalSkew(call_or_put,df_tmp,greeks='Delta'):  #这个CalSkew是有问题的
    want_greeks_num = 0.25*call_or_put
    TC1_new = FindDelta(want_greeks_num=want_greeks_num,df=df_tmp,greeks='Delta')
    x1 = abs(TC1_new[greeks][1]-want_greeks_num)
    x2 = abs(TC1_new[greeks][0]-want_greeks_num)
    x11 = x1/(x1+x2)
    x22 = x2/(x1+x2)
    IV_25= TC1_new['ImpliedVolatility'][0]*x11 + TC1_new['ImpliedVolatility'][1]*x22
    want_greeks_num = 0.5*call_or_put
    TC2_new = FindDelta(want_greeks_num=want_greeks_num,df=df_tmp,greeks='Delta')
    x1 = abs(TC2_new[greeks][1]-want_greeks_num)
    x2 = abs(TC2_new[greeks][0]-want_greeks_num)
    x11 = x1/(x1+x2)
    x22 = x2/(x1+x2)
    IV_50= TC2_new['ImpliedVolatility'][0]*x11 + TC2_new['ImpliedVolatility'][1]*x22
    skew = IV_25/IV_50
    return(skew)
    
def CalSkewV2(call_or_put,df_tmp,greeks='Delta'):  #这是张扬6月26日新修改的
    want_greeks_num = 0.25*call_or_put
    TC1_new = FindDelta(want_greeks_num=want_greeks_num,df=df_tmp,greeks='Delta')
    x1 = TC1_new[greeks][1]
    x0 = TC1_new[greeks][0]
    y1=TC1_new['ImpliedVolatility'][1]  #这里这种写法纯属为了看的清楚
    y0=TC1_new['ImpliedVolatility'][0]
    IV_25= (want_greeks_num-x1)*(y0-y1)/(x0-x1)+y1
    want_greeks_num = 0.5*call_or_put
    TC2_new = FindDelta(want_greeks_num=want_greeks_num,df=df_tmp,greeks='Delta')
    x1 = TC2_new[greeks][1]
    x0 = TC2_new[greeks][0]
    y1=TC2_new['ImpliedVolatility'][1]  #这里这种写法纯属为了看的清楚
    y0=TC2_new['ImpliedVolatility'][0]
    IV_50= (want_greeks_num-x1)*(y0-y1)/(x0-x1)+y1
    skew = IV_25/IV_50
    return(skew)

def CalSkewV3(call_or_put,df_tmp,greeks='Delta'):  #V2与V3本质上是完全相同的
    want_greeks_num = 0.25*call_or_put
    TC1_new = FindDelta(want_greeks_num=want_greeks_num,df=df_tmp,greeks='Delta')
    x1 = want_greeks_num-TC1_new[greeks][1]
    x0 = TC1_new[greeks][0]-want_greeks_num
    x11 = x1/(x0+x1)
    x00 = x0/(x0+x1)
    IV_25= TC1_new['ImpliedVolatility'][0]*x11 + TC1_new['ImpliedVolatility'][1]*x00
    want_greeks_num = 0.5*call_or_put
    TC2_new = FindDelta(want_greeks_num=want_greeks_num,df=df_tmp,greeks='Delta')
    x1 = want_greeks_num-TC2_new[greeks][1]
    x0 = TC2_new[greeks][0]-want_greeks_num
    x11 = x1/(x0+x1)
    x00 = x0/(x0+x1)
    IV_50= TC2_new['ImpliedVolatility'][0]*x11 + TC2_new['ImpliedVolatility'][1]*x00
    skew = IV_25/IV_50
    return(skew)
    
#截止到20190626日 V4这个版本的CalSkew是最完善的
def CalSkewV4(call_or_put,df_tmp,greeks='Delta'):  #V4 要求一个点Delta是比0.25大的，一个点Delta比0.25小
    want_greeks_num = 0.25*call_or_put
    TC1_new = FindDeltaV3(want_greeks_num=want_greeks_num,df=df_tmp,greeks='Delta')  #使用FindDeltaV3
    Flag1=0
    if TC1_new.shape[0]>2:
        for i in range(TC1_new.shape[0]):
            if TC1_new[greeks][i] != TC1_new[greeks][0]:
                delta1_x1=TC1_new[greeks][i]
                TC1_index=i
                break
            if i==TC1_new.shape[0]:
                Flag1=1   #代表非常特殊的情况，就是所有Call合约的Delta值都相同
    #    x1 = want_greeks_num-TC1_new[greeks][1]  #x1是delta比较小的
        if Flag1 ==0:
            x1 = want_greeks_num-delta1_x1
            x0 = TC1_new[greeks][0]-want_greeks_num  #x0是delta比较大的
            x11 = x1/(x0+x1)
            x00 = x0/(x0+x1)
            IV_25= TC1_new['ImpliedVolatility'][0]*x11 + TC1_new['ImpliedVolatility'][TC1_index]*x00
        else:
            IV_25=1.0
    else:
        x1 = want_greeks_num-TC1_new[greeks][1]  #x1是delta比较小的
        x0 = TC1_new[greeks][0]-want_greeks_num  #x0是delta比较大的
        x11 = x1/(x0+x1)
        x00 = x0/(x0+x1)
        IV_25= TC1_new['ImpliedVolatility'][0]*x11 + TC1_new['ImpliedVolatility'][1]*x00
    
    want_greeks_num = 0.5*call_or_put
    TC2_new = FindDeltaV3(want_greeks_num=want_greeks_num,df=df_tmp,greeks='Delta')
    Flag2=0
    if TC2_new.shape[0]>2:
        for i in range(TC2_new.shape[0]):
            if TC2_new[greeks][i] != TC1_new[greeks][0]:
                delta2_x1=TC2_new[greeks][i]
                TC2_index=i
                break
            if i==TC2_new.shape[0]:
                Flag2=1   #代表非常特殊的情况，就是所有Call合约的Delta值都相同
        if Flag2 ==0:
            x1 = want_greeks_num-delta2_x1
            x0 = TC2_new[greeks][0]-want_greeks_num
            x11 = x1/(x0+x1)
            x00 = x0/(x0+x1)
            IV_50= TC2_new['ImpliedVolatility'][0]*x11 + TC2_new['ImpliedVolatility'][TC2_index]*x00
        else:
            IV_50=1.0
    else:
        x1 = want_greeks_num-TC2_new[greeks][1]
        x0 = TC2_new[greeks][0]-want_greeks_num
        x11 = x1/(x0+x1)
        x00 = x0/(x0+x1)
        IV_50= TC2_new['ImpliedVolatility'][0]*x11 + TC2_new['ImpliedVolatility'][1]*x00 
    skew = IV_25/IV_50
    return(skew)


def CalSkewV5(call_or_put,df_tmp,greeks='DELTA'):  #V4 要求一个点Delta是比0.25大的，一个点Delta比0.25小
    want_greeks_num = 0.25*call_or_put
    TC1_new = FindDeltaV3(want_greeks_num=want_greeks_num,df=df_tmp,greeks='DELTA')  #使用FindDeltaV3
    Flag1=0
    if TC1_new.shape[0]>2:
        for i in range(TC1_new.shape[0]):
            if TC1_new[greeks][i] != TC1_new[greeks][0]:
                delta1_x1=TC1_new[greeks][i]
                TC1_index=i
                break
            if i==TC1_new.shape[0]:
                Flag1=1   #代表非常特殊的情况，就是所有Call合约的Delta值都相同
    #    x1 = want_greeks_num-TC1_new[greeks][1]  #x1是delta比较小的
        if Flag1 ==0:
            x1 = want_greeks_num-delta1_x1
            x0 = TC1_new[greeks][0]-want_greeks_num  #x0是delta比较大的
            x11 = x1/(x0+x1)
            x00 = x0/(x0+x1)
            IV_25= TC1_new['Implied Volatility'][0]*x11 + TC1_new['Implied Volatility'][TC1_index]*x00
        else:
            IV_25=1.0
    else:
        x1 = want_greeks_num-TC1_new[greeks][1]  #x1是delta比较小的
        x0 = TC1_new[greeks][0]-want_greeks_num  #x0是delta比较大的
        x11 = x1/(x0+x1)
        x00 = x0/(x0+x1)
        IV_25= TC1_new['Implied Volatility'][0]*x11 + TC1_new['Implied Volatility'][1]*x00
    
    want_greeks_num = 0.5*call_or_put
    TC2_new = FindDeltaV3(want_greeks_num=want_greeks_num,df=df_tmp,greeks='DELTA')
    Flag2=0
    if TC2_new.shape[0]>2:
        for i in range(TC2_new.shape[0]):
            if TC2_new[greeks][i] != TC2_new[greeks][0]:
                delta2_x1=TC2_new[greeks][i]
                TC2_index=i
                break
            if i==TC2_new.shape[0]:
                Flag2=1   #代表非常特殊的情况，就是所有Call合约的Delta值都相同
        if Flag2 ==0:
            x1 = want_greeks_num-delta2_x1
            x0 = TC2_new[greeks][0]-want_greeks_num
            x11 = x1/(x0+x1)
            x00 = x0/(x0+x1)
            IV_50= TC2_new['Implied Volatility'][0]*x11 + TC2_new['Implied Volatility'][TC2_index]*x00
        else:
            IV_50=1.0
    else:
        x1 = want_greeks_num-TC2_new[greeks][1]
        x0 = TC2_new[greeks][0]-want_greeks_num
        x11 = x1/(x0+x1)
        x00 = x0/(x0+x1)
        IV_50= TC2_new['Implied Volatility'][0]*x11 + TC2_new['Implied Volatility'][1]*x00 
    skew = IV_25/IV_50
    return(skew,IV_25)
'''   
import time


start = time.clock()

#your code here
     
S=2.3352 #for stock
X=2.3   #for strike
T=0.0328
r=0.0000
c=0.03925
p=0.00505
IV1=CalIVCall(S,X,T,r,c)
IV2=CalIVPut(S,X,T,r,p)


print(IV1)

print(IV2)

DeltaCall=CalDeltaCall(S,X,T,r,IV1)
DeltaPut=CalDeltaPut(S,X,T,r,IV2)
GammaCall=CalGammaCall(S,X,T,r,IV1)
GammaPut=CalGammaPut(S,X,T,r,IV2)


print('DeltaCall',DeltaCall)
print('DeltaPut',DeltaPut)
print('GammaCall',GammaCall)
print('GammaPut',GammaPut)


#print(IV2)

end = time.clock()
print( 'time used:',(end-start),'seconds')    
'''




from setuptools import setup, find_packages

setup(
    name='dwj_tools', # 项目的名称,pip3 install get-time
    version='0.0.4', # 项目版本 
    author='丁文杰', # 项目作者 
    author_email='359582058@qq.com', # 作者email 
    url='https://github.com/Coxhuang/get_time', # 项目代码仓库
    description='私人', # 项目描述 
    packages=find_packages(), # 包名 
    install_requires=[],
    entry_points={
        'console_scripts': [
            'get_time=get_time:get_time', # 使用者使用get_time时,就睡到get_time项目下的__init__.py下执行get_time函数
            'get_timestamp=get_time:get_timestamp',
            'timestamp_to_str=get_time:timestamp_to_str',
            'str_to_timestamp=get_time:str_to_timestamp',
        ]
    } # 重点
)




from tcorepyapi.tcoreapi_mq import * 
import tcorepyapi.tcoreapi_mq
import datetime

#登入
TCoreAPI= TCoreZMQ(quote_port="51630",trade_port="51600")


#订阅合约实时行情报价
#订阅单个合约参数str或list
TCoreAPI.SubQuote("TC.F.SHFE.rb.202110")
#订阅多个合约list
#TCoreAPI.SubQuote(["TC.F.SHFE.rb.202110","TC.F.CFFEX.IF.202106"])

#订阅合约的Greeks实时数据
#订阅单个合约参数str或list,订阅多个合约list
TCoreAPI.SubGreeks(["TC.F.U_SSE.510050.202106"])

while True:
    message=TCoreAPI.mdupdate()
    if message:
        if message['DataType']=='REALTIME':
            print("实时行情 \n 合约：",datetime.datetime.now(),message['Quote']['Symbol'],"  ",int(message['Quote']['FilledTime'])+80000,"  当日成交量：%s" % message['Quote']['TradeVolume'])
        elif  message['DataType']=="GREEKS":
            print("实时GREEKS \n IV:",datetime.datetime.now(),message['Quote'])
